// Thomas Grenier - Creatis

function [gfx]=DParzen_GG(X,Xi,h,trunc)
// Xi : echantillon des données : matrice
// X  : point où réaliser l'estimation
// GG : version Généralisée (n dimensions, spatial en 1 et 2)
// h : parametre d'echelle pas mis au carre !!
// trunc : if <> 0 trunc Gaussian to 3.sigma (3.h)
// fx : estimation au point X

PI=3.141592;
NORM_GAUSS = 5.0132565; // (2*sqrt(2*PI));
trunc2 = trunc*trunc;

[nb_Xi,nb_Dim] = size(Xi); 

// Profile Gaussien -derive g=-k'
function [y]=g_gauss(x)
y = exp(-x/2) / NORM_GAUSS;
endfunction

SN=0;
gfx=zeros(1,nb_Dim);
ih1 = 1.0/(h(1)*h(1));
ih2 = 1.0/(h(2)*h(2));
ih3 = 1.0/(h(3)*h(3));

iHr = ih3*eye(nb_Dim-2, nb_Dim-2);

for i = 1:nb_Xi,
  x_xi = X  - Xi(i,:);

  dxs = x_xi(1)*x_xi(1)*ih1 + x_xi(2)*x_xi(2)*ih2;
  dxr = x_xi(3:nb_Dim)*iHr*x_xi(3:nb_Dim)';
  
  if(trunc == 0 ) then
      gxs = g_gauss( dxs );
      gxr = g_gauss( dxr );
      N = gxs*gxr;
      gfx = gfx + Xi(i,:)*N;
      SN = SN + N;
  end

  if (trunc ~= 0 & dxs < trunc2 & dxr < trunc2 ) then
      gxs = g_gauss( dxs );
      gxr = g_gauss( dxr );
      N = gxs*gxr;
      gfx = gfx + Xi(i,:)*N;
      SN = SN + N;
  end
end

if SN~=0,
  gfx = gfx / SN;
end

endfunction


