// Thomas Grenier - Creatis

PI  = 3.141592;

function [ms_f1, Y] = FiltrageMeanShift_1f( Image_f1, n, h)
// Image : feature image d'entree
// n : taille de l'echantillon pour l'estimation 
// h : vecteur de parametres d'echelle

// Shoot Xi  qui s'appellent Y ...
[nb_X, nb_Y] = size(Image_f1);
Y = [grand( n , 1 , 'unf', 1, nb_X ), grand( n , 1 , 'unf', 1, nb_Y ), ones(n,1) ];
for i=1:n,
  Y(i,3) = Image_f1( Y(i,1), Y(i,2) );
end

ms_f1 = Image_f1;
invH = inv( diag( h.*h ) );

for i=1:nb_X,
  printf('%d ',i);
  for j=1:nb_Y,
    a = ProcedureMeanShift_1f( [i , j, Image_f1(i,j)] , Y, invH, 0.1);
    ms_f1(i,j) = a(3);
  end;
end;
endfunction;


function d=Dist2Mahalanobis( a, b, invH)
  dx = a - b;
  d = dx*invH*dx';
endfunction;


function [x_n]=ProcedureMeanShift_1f( x, Xi, invH, epsilon)
// x : point à deplacer
// Xi : echantillon pour l'estimation
// h : vecteur d'echelles
// epsilon : distance mini accpetable
// x_n : resultat : x à l'iteration suivante
x_tmp = x;
x_n = DParzen_G_1f( x_tmp, Xi, invH);
d   = Dist2Mahalanobis( x_tmp, x_n, invH)
// calcul deplacement 
while  d > epsilon,
  x_n = DParzen_G_1f( x_tmp, Xi, invH);
  d   = Dist2Mahalanobis( x_tmp, x_n, invH)
  x_tmp = x_n;
end;  
endfunction;

// Profile Gaussien (k)
function [y] = k_gauss(x)
y = exp(-x/2) / sqrt(2*PI);
endfunction

// Profile Gaussien -derive g=-k'
function [y] = g_gauss(x)
y = exp(-x/2) / (2*sqrt(2*PI));
endfunction

function [gfx] = DParzen_G_1f( X, Xi, invH)
// Xi : echantillon des données : matrice
// X  : point où réaliser l'estimation
// fx : estimation au point X

[nb_Xi,nb_Dim] = size(Xi); 
//printf('nb_Dim : %d; nb_Xi : %d \n', nb_Dim, nb_Xi)

SN  = zeros(1,nb_Dim);
gfx = zeros(1,nb_Dim);

invh = diag( invH );

for i = 1:nb_Xi,
  x_xi = X  - Xi(i,:);

  dxs = x_xi(1)*x_xi(1)*h(1) + x_xi(2)*x_xi(2)*h(2);
  dxr = x_xi(3)*x_xi(3)*h(3);
 
  kxs = k_gauss( dxs ); 
  kxr = k_gauss( dxr );  
  gxs = g_gauss( dxs );
  gxr = g_gauss( dxr );

  N = [gxs*kxr, gxs*kxr, gxr*kxs ];
  
  gfx = gfx + Xi(i,:).*N;
  
  SN = SN + N;
end

for i=1:nb_Dim,
  if SN(i)<>0,
    gfx(i) = gfx(i) / SN(i);
  end
end

endfunction
