// Thomas Grenier - Creatis

// Ps : spatial position of all sample of the population (the whole data set to be filtered)    
// Pr : range  position of all sample of the population (the whole data set to be filtered)    
// nb_Xi : number of samples to be randomilly choosen in the population
// hs : spatial bandwidth parameter (vector of h values)
// hr : range bandwidth parameter (vector of h values)
function [FPs, FPr]=GBMS_f(nb_Xi, Ps, Pr, hs, hr, eps_ms, nb_max_it, trunc_gauss)

    [nb_Ps, nb_DimS] = size(Ps);
    [nb_Pr, nb_DimR] = size(Pr);
    
    FPs = zeros(Ps);
    FPr = zeros(Pr);
    
    Ci = 0;
    if( nb_Ps ~= nb_Pr) disp('the population sets Ps and Pr should have the same number of entries');
        return(Ci);
    end
    
     if( size(hs) ~= nb_DimS) disp('the dimension of hs and Ps should be the same');
        return(Ci);
     end
   
     if( size(hr) ~= nb_DimR) disp('the dimension of hr and Pr should be the same');
        return(Ci);
     end
    
    // GMS 
    sample_index = grand( nb_Xi , 1 , 'uin', 1, nb_Ps );
    [As, Ar] = GBMS( Ps(sample_index,:), Pr(sample_index,:), hs, hr, eps_ms, nb_max_it, trunc_gauss);
    
    // Affecting value to unprocessed samples
    iHs = inv(diag(hs.^2));
//    hr = hr / 5;
    iHr = inv(diag(hr.^2));

    d2  = zeros(nb_Xi,3);
    for i=1:nb_Ps
        disp('GBMS_f It :'+ string(i) + ' of  ' + string(nb_Ps));
        for j=1:nb_Xi
            x_xi = Ps(sample_index(j),:) - Ps(i,:); 
            d2xs = x_xi*iHs*x_xi';
           
            x_xi = Pr(sample_index(j),:) - Pr(i,:); 
            d2xr = x_xi*iHr*x_xi';
           
            d2(j,:) = [d2xs + d2xr, d2xs, d2xr];              
        end
        
        min_index = find( d2 == min(d2(:,1)) );
        min_index = min_index(1);
        FPs(i,:) = As(min_index,:);
        FPr(i,:) = Ar(min_index,:);
    end

endfunction
