function [fx]=Parzen_G1D_B_t( X, Xi, h, Low, High)
//  Methods for Removing Boundary Effects :   Reflection Data Method optimized
// Xi : echantillon des données : vecteur de scalaire (1D...)
// X  : point où réaliser l'estimation (scalaire aussi)
// Low : lower bound of estimation
// High : upper bound
// fx : estimation au point X


PI=3.141592;
[nb_Xi, nb_Dim] = size(Xi); 
//disp(nb_Xi);
//disp( nb_Dim);

// Profile Gaussien
function [y]=k_gauss(x)
    y = exp(-x/2) / sqrt(2*PI);
endfunction

if (nb_Dim <> 1) then
    disp("probable mistake on Xi: should be n by 1 vector");
end

fx=0;

if (X >= Low) & (X <= High) then
       
    for i = 1:nb_Xi,
        x_xi = X - Xi(i);
        x_xi_l = Low - X - Xi(i);
        x_xi_h = 2*High - X - Xi(i);
        
        dx   = x_xi*x_xi/(h*h);
        dx_l = x_xi_l*x_xi_l/(h*h);
        dx_h = x_xi_h*x_xi_h/(h*h);
        
        fx = fx + k_gauss( dx ) + k_gauss( dx_l ) + k_gauss( dx_h ); 
    end
    
end

if fx <> 0 then
    fx = fx / (nb_Xi * h); // 1/2 with '+' in fx = fx ...
end

endfunction


// TESTING
// Xi = grand(200,1,"unf", 0, 20); // uniforme
//Xi = [0:0.5:20]'; // uniforme de chez uniforme...
//x =[-7:0.1:28]';
//for i=1:size(x,1), f_B(i)   = Parzen_G1D_B( x(i), Xi, 1, 0, 20); end;
//for i=1:size(x,1), f(i)     = Parzen_G1D( x(i), Xi, 1); end;
//for i=1:size(x,1), f_B_t(i) = Parzen_G1D_B_t( x(i), Xi, 1, 0, 20); end;
//sum(f)   // attention à l'echantillonnage
//sum(f_B) // take care of sampling rate ...
//sum(f_B_t)
//plot(x,f, 'b', x,f_B,'r', x, f_B_t, 'g');
//legend(["Classical Parzen" "Parzen Bounded under weight" "Parzen Bounded Optimized"]);
//xtitle("Loi uniforme parfaite","X","Fréquence");
//
////
//    lambda=1.6;
//    N=10000;
//    Xi = grand(N,1,"exp",lambda);
//    scf();
//    classes = linspace(-1,12,25);
//    histplot(classes,Xi)
//    x=linspace(-1,12,25);
//    y = (1/lambda)*exp(-(1/lambda)*x);
//    plot(x,y,"ro-");
//    f_B_t = 0;
//    for i=1:size(x,2), f_B_t(i) = Parzen_G1D_B_t( x(i), Xi, 1, 0, 200); end;
//    plot(x,f_B_t, 'g');
//    f = 0;
//    for i=1:size(x,2), f(i)     = Parzen_G1D( x(i), Xi, 1); end;    
//    plot(x,f, 'b');
//    legend(["Empirique" "Theorique" "Parzen Bounded Optimized" "Classical Parzen"]);
//    xtitle("Loi exponentielle par grand","X","Fréquence");
//    
//
